//+---------------------------------------------------------------------------------------------+
//| ATR Morning Breakout: Trades all fractal zone breakouts when the ATR is above its MA        |
//+---------------------------------------------------------------------------------------------+

#define SIGNAL_NONE 0
#define SIGNAL_BUY   1
#define SIGNAL_SELL  2
#define SIGNAL_CLOSEBUY 3
#define SIGNAL_CLOSESELL 4

#property copyright "Alexander C. Boyd"

extern string  BasicSettings        = "=======> EA Settings";
extern int     SessionEarly         = 7;
extern int     SessionLate          = 12;
extern int     MagicNumber          = 1918271;
extern int     Slippage             = 3;
extern bool    EachTickMode         = true;
extern bool    SignalMail           = False;

extern string  IndicatorSettings    = "=======> Indicator Settings";
extern int     ATRPeriod            = 16;
extern int     BaseSMA              = 3;
extern int     SignalSMA            = 14;

extern string  MoneyManagement            = "=======> Money Management Settings";
extern bool    UseProportionalLotSizing   = true;  // Use equity as a gauge of lot size
extern double  RiskFactor                 = 5;     // Risk level
extern double  Lots                       = 0.1;   // If using static lot sizing

extern string  OrderManagement            = "=======> Order Settings";
extern bool    UseDynamicTargets          = true;
extern double  StopLossMultiplier         = 0.9;
extern double  TakeProfitMultiplier       = 0.75;
extern bool    UseTrailingStop            = false;
extern int     TrailingStop               = 185;
extern bool    UseStopLoss                = false;
extern int     StopLoss                   = 200;
extern bool    UseTakeProfit              = false;
extern int     TakeProfit                 = 500;

int BarCount;
int Current;
bool TickCheck = False;


//+-----------------+
//| Initialization  |
//+-----------------+

int init() {BarCount = Bars; if (EachTickMode) Current = 0; else Current = 1; return(0);}
int deinit() {return(0);}

int start() {

   int Order = SIGNAL_NONE;
   int Ticket;
   double StopLossLevel, TakeProfitLevel;
   if (EachTickMode && Bars != BarCount) TickCheck = False;
   int Total = OrdersTotal();
   Order = SIGNAL_NONE;
   
//---------- Define variables and such

double   ATRMA             = iCustom(NULL, 0, "ATR MA", ATRPeriod, BaseSMA, SignalSMA, 1, 0);
double   ATRSignal         = iCustom(NULL, 0, "ATR MA", ATRPeriod, BaseSMA, SignalSMA, 2, 0);

double   ATRMA_Shift       = iCustom(NULL, 0, "ATR MA", ATRPeriod, BaseSMA, SignalSMA, 1, 1);
double   ATRSignal_Shift   = iCustom(NULL, 0, "ATR MA", ATRPeriod, BaseSMA, SignalSMA, 2, 1);

double   CloseLastBar      = iClose(NULL, 0, 1);
double   CloseTwoBars      = iClose(NULL, 0, 2);

static double     HighFractal;
static double     LowFractal;

static datetime   HighFractalTime;
static datetime   LowFractalTime;

if (ATRMA > ATRSignal) bool HighRange = true;

//---------- Assign ATR Crossing Times

if (ATRMA_Shift > ATRSignal_Shift && ATRMA < ATRSignal)  // Down cross time
      {
         static datetime   ATRCrossDownTime; ATRCrossDownTime = TimeCurrent();
      }
      
if (ATRMA_Shift < ATRSignal_Shift && ATRMA > ATRSignal && Hour() > SessionEarly && Hour() < SessionLate)  // Up cross time
      {
         static datetime   ATRCrossUpTime; ATRCrossUpTime = TimeCurrent();
      }
         int LowVolumeTime = MathAbs(ATRCrossUpTime - ATRCrossDownTime);
         if ((ATRCrossUpTime - ATRCrossDownTime) > 0) {
            
            int LowVolumeBars = MathRound(LowVolumeTime/Period());} // Get number of bars since last down cross

//---------- Assign High/Low Fractal Values

   for(int k = 1; k <= LowVolumeBars; k++){   // Get last high fractal
      if(iFractals(NULL, Period(), MODE_UPPER,k)!=0){
         HighFractal = iFractals(NULL, Period(), MODE_UPPER,k);
         HighFractalTime = Time[k];
         break;
         }
      }

   for(int j = 1; j <= LowVolumeBars; j++){   // Get last low fractal
      if(iFractals(NULL, Period(), MODE_LOWER,j)!=0){
         LowFractal = iFractals(NULL, Period(), MODE_LOWER,j);
         LowFractalTime = Time[j];
         break;
         }
      }

//---------- Money Management

      if (UseProportionalLotSizing){double Lots = NormalizeDouble(AccountEquity()*(0.00001*RiskFactor), 2);
      if (Lots < 0.01) Lots = MODE_MINLOT;}

         int Spread = MarketInfo("EURUSD",MODE_SPREAD);

      if (UseDynamicTargets)
      {
            UseStopLoss = true; UseTakeProfit = true; double FractalRangePips = (HighFractal - LowFractal)/Point;
            StopLoss = MathRound(StopLossMultiplier*FractalRangePips+Spread);
            TakeProfit = MathRound(TakeProfitMultiplier*FractalRangePips+Spread);
      }

//------- How to close trades

   //Check position
   bool IsTrade = False;

   for (int i = 0; i < Total; i ++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderType() <= OP_SELL &&  OrderSymbol() == Symbol()) {
         IsTrade = True;
         if(OrderType() == OP_BUY) {


//------- Exit buy signal

        if (!HighRange) Order = SIGNAL_CLOSEBUY;

//------- Exit buy [end]


            if (Order == SIGNAL_CLOSEBUY && ((EachTickMode && !TickCheck) || (!EachTickMode && (Bars != BarCount)))) {
               OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, MediumSeaGreen);
               if (SignalMail) SendMail("[Signal Alert]", "[" + Symbol() + "] " + DoubleToStr(Bid, Digits) + " Close Buy");
               if (!EachTickMode) BarCount = Bars;
               IsTrade = False;
               continue;
            }
            //Trailing stop
            if(UseTrailingStop && TrailingStop > 0) {                 
               if(Bid - OrderOpenPrice() > Point * TrailingStop) {
                  if(OrderStopLoss() < Bid - Point * TrailingStop) {
                     OrderModify(OrderTicket(), OrderOpenPrice(), Bid - Point * TrailingStop, OrderTakeProfit(), 0, MediumSeaGreen);
                     if (!EachTickMode) BarCount = Bars;
                     continue;
                  }
               }
            }
         } else {


//------- Exit sell signal

        if (!HighRange) Order = SIGNAL_CLOSESELL;

//------- Exit sell [end]


            if (Order == SIGNAL_CLOSESELL && ((EachTickMode && !TickCheck) || (!EachTickMode && (Bars != BarCount)))) {
               OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, DarkOrange);
               if (SignalMail) SendMail("[Signal Alert]", "[" + Symbol() + "] " + DoubleToStr(Ask, Digits) + " Close Sell");
               if (!EachTickMode) BarCount = Bars;
               IsTrade = False;
               continue;
            }
            //Trailing stop
            if(UseTrailingStop && TrailingStop > 0) {                 
               if((OrderOpenPrice() - Ask) > (Point * TrailingStop)) {
                  if((OrderStopLoss() > (Ask + Point * TrailingStop)) || (OrderStopLoss() == 0)) {
                     OrderModify(OrderTicket(), OrderOpenPrice(), Ask + Point * TrailingStop, OrderTakeProfit(), 0, DarkOrange);
                     if (!EachTickMode) BarCount = Bars;
                     continue;
                  }
               }
            }
         }
      }
   }

//------- CONDITIONS FOR OPENING TRADES

int LastTrade = HistoryTotal();
    if(LastTrade > 0)   {
        if(OrderSelect(LastTrade,SELECT_BY_POS,MODE_HISTORY) == true)
            int LastOpenTime = OrderOpenTime();
                        }

     
     if (LastOpenTime > ATRCrossUpTime) bool NoMoreTradesToday = true;


if (!NoMoreTradesToday && HighRange && CloseLastBar < HighFractal && Ask > HighFractal) Order = SIGNAL_BUY;  // Place order if price moves outside of the breakout range from the inside of it
if (!NoMoreTradesToday && HighRange && CloseLastBar > LowFractal && Bid < LowFractal) Order = SIGNAL_SELL;


//------- Enter buy

   if (Order == SIGNAL_BUY && ((EachTickMode && !TickCheck) || (!EachTickMode && (Bars != BarCount)))) {
      if(!IsTrade) {
         //Check free margin
         if (AccountFreeMargin() < (1000 * Lots)) {
            Print("We have no money. Free Margin = ", AccountFreeMargin());
            return(0);
         }

         if (UseStopLoss) StopLossLevel = Ask - StopLoss * Point; else StopLossLevel = 0.0;
         if (UseTakeProfit) TakeProfitLevel = Ask + TakeProfit * Point; else TakeProfitLevel = 0.0;

         Ticket = OrderSend(Symbol(), OP_BUY, Lots, Ask, Slippage, StopLossLevel, TakeProfitLevel, "Buy(#" + MagicNumber + ")", MagicNumber, 0, DodgerBlue);
         if(Ticket > 0) {
            if (OrderSelect(Ticket, SELECT_BY_TICKET, MODE_TRADES)) {
				Print("BUY order opened : ", OrderOpenPrice());
                if (SignalMail) SendMail("[Signal Alert]", "[" + Symbol() + "] " + DoubleToStr(Ask, Digits) + " Open Buy");
			} else {
				Print("Error opening BUY order : ", GetLastError());
			}
         }
         if (EachTickMode) TickCheck = True;
         if (!EachTickMode) BarCount = Bars;
         return(0);
      }
   }
   
//------- Enter sell

   if (Order == SIGNAL_SELL && ((EachTickMode && !TickCheck) || (!EachTickMode && (Bars != BarCount)))) {
      if(!IsTrade) {
         //Check free margin
         if (AccountFreeMargin() < (1000 * Lots)) {
            Print("We have no money. Free Margin = ", AccountFreeMargin());
            return(0);
         }

         if (UseStopLoss) StopLossLevel = Bid + StopLoss * Point; else StopLossLevel = 0.0;
         if (UseTakeProfit) TakeProfitLevel = Bid - TakeProfit * Point; else TakeProfitLevel = 0.0;

         Ticket = OrderSend(Symbol(), OP_SELL, Lots, Bid, Slippage, StopLossLevel, TakeProfitLevel, "Sell(#" + MagicNumber + ")", MagicNumber, 0, DeepPink);
         if(Ticket > 0) {
            if (OrderSelect(Ticket, SELECT_BY_TICKET, MODE_TRADES)) {
				Print("SELL order opened : ", OrderOpenPrice());
                if (SignalMail) SendMail("[Signal Alert]", "[" + Symbol() + "] " + DoubleToStr(Bid, Digits) + " Open Sell");
			} else {
				Print("Error opening SELL order : ", GetLastError());
			}
         }
         if (EachTickMode) TickCheck = True;
         if (!EachTickMode) BarCount = Bars;
         return(0);
      }
   }
   
   if (!EachTickMode) BarCount = Bars;
   
if(TimeCurrent() > HighFractalTime+86400){   // Specify that fractals are good for one day only
  HighFractal = 0; HighFractalTime = 0;}
  
if(TimeCurrent() > LowFractalTime+86400){
  LowFractal = 0; LowFractalTime = 0;}

//------- Comment relevant/important information

double FreeMargin = NormalizeDouble(AccountFreeMargin(),2);
double TotalProfit = NormalizeDouble(AccountProfit(),2);
double UpperFractal = NormalizeDouble(HighFractal,Digits);
double LowerFractal = NormalizeDouble(LowFractal,Digits);

Comment(
        "Bars Since Last ATR Cross: "+LowVolumeBars+" \n",
        "Upper fractal breakout level: "+UpperFractal+" \n",
        "Lower fractal breakout level: "+LowerFractal+" \n",
        "Spread: "+Spread+" \n",
        "\n",
        "Available capital: "+FreeMargin+" \n",
        "Floating P/L: "+TotalProfit+" \n"
        );

   return(0);
}

//------- END